@@ -8,5 +8,5 @@
 extern "C"
 {
 	const char *RPiCamAppsVersion();
-	const char *RPiCamAppsCapabilities();
+	const char *RPiCamAppsCapabilities(const std::string &preview_libs, const std::string &encoder_libs);
 }
